import requests
import base64
import json
from urllib.parse import urljoin, urlencode

class GiteaConnector:
    def __init__(self, repo_url, user, key):
        self.repo_url = repo_url
        self.user = user
        self.key = key

        self.header = {
            "accept": "application/json",
            "Authorization": f"token {self.key}",
            "Content-Type": "application/json",
        }

    ## Get open issues in repo
    def get_open_issue_titles(self):
        url_suffix = "issues"
        url = urljoin(self.repo_url, url_suffix)
        queries = {
            "state": "open",
            "type": "issues",
        }

        full_url = f"{url}?{urlencode(queries)}"
        response = requests.get(full_url, headers=self.header)
        if response.status_code == 200:
            try:
                data = response.json()  # Parse JSON response
                if isinstance(data, list):  # Confirm it's a list
                    # Extract 'title' from each item in the list
                    existing_titles = [item['title'] for item in data if 'title' in item]
                    return existing_titles
                else:
                    print("Unexpected response format: Expected a list.")
            except ValueError:
                print("Error: Response is not valid JSON.")
        else:
            print("Error fetching existing issues")
        return []


    ## Create an issue in the repo
    def create_issue(self,title,content):
        url_suffix = "issues"
        url = urljoin(self.repo_url, url_suffix)
        existing_issue_titles = self.get_open_issue_titles()
        if title in existing_issue_titles:
            print("Issue regarding this code exists already")
            return True
        issue_data = {
            "title": title,
            "body": content,
            "assignees": ["fred.boniface"],
            "labels": [261,269],
        }

        print(f"URL: {url}")
        response = requests.post(url, headers=self.header, json=issue_data)
        if response.status_code == 201:
            print("Succesfully created issue")
            return True
        else:
            print("Failed to create issue: ", response.status_code, response.text)
            return False

    ## Download the 'gw' pis file from the repo
    def download_pis_file(self):
        url = urljoin(self.repo_url, 'contents/pis/gw.yaml')
        response = requests.get(url, headers=self.header)
        if response.status_code == 200:
            file_content = base64.b64decode(response.json()['content']).decode('utf-8')
            print("File downloaded successfully")
            return file_content
        else:
            print("Failed to download file: ", response.status_code, response.text)
            return ""
        
    ## Create new branch in the repo
    def create_branch(self, branch_name):
        BASE_BRANCH = "main"

        create_branch_url = urljoin(self.repo_url, "branches")
        branch_data = {
            "new_branch_name": branch_name,
            "old_branch_name": BASE_BRANCH,
        }

        create_branch_response = requests.post(create_branch_url, headers=self.header, data=json.dumps(branch_data))
        if create_branch_response.status_code == 201:
            print(f"Branch {branch_name} created successfully")
            return True
        else:
            print(f"Failed to create branch: ", branch_name, create_branch_response.status_code, create_branch_response.text)
            return False

    ## Create a new file in the repo
    def create_pis_file(self, branch, filename, file_content):
        file_path = f"pis/{filename}"
        encoded_content = base64.b64encode(file_content.encode('utf-8')).decode('utf-8')

        create_file_url = urljoin(self.repo_url, f'contents/{file_path}.yaml')

        file_data = {
            "content": encoded_content,
            "message": "Create new PIS File",
            "branch": branch,
        }

        create_file_response = requests.post(create_file_url, headers=self.header, data=json.dumps(file_data))

        if create_file_response.status_code == 201:
            print("File created successfully")
            return True
        else:
            print("File creation failed.", create_file_response.status_code, create_file_response.text)
            return False